MCU=atmega168
CC=avr-gcc -std=c99
OBJCOPY=avr-objcopy
CFLAGS=-ffunction-sections -mmcu=$(MCU) -Wall -Wextra -Wpadded -Wshadow -Winline -Wdisabled-optimization -Wstrict-prototypes -Waggregate-return -Os -fshort-enums -mcall-prologues -DNDEBUG

main.hex : main.elf
	$(OBJCOPY) -R .eeprom -O ihex $^ $@
	SIZE=
	date +"%s`avr-size $^ | tail -n 1 | sed "s/[[:space:]]\+/, /g"`" >> size.log
	avr-size *.o $^
	@echo " "
	@echo "data=initialized data, bss=uninitialized data, text=code"
	@echo " "

eeprom.hex : main.elf
	$(OBJCOPY) -j .eeprom --change-section-lma .eeprom=0x0 -O ihex $^ $@

CFLAGS+=-DENABLE_MEMDUMP -g
main.elf : main.o seq.o pwm.o enc28j60.o arp.o irq.o autoip.o timer.o ip.o icmp.o udp.o dhcp.o eth.o netbios.o light.o tcp.o eprom.o stackmon.o memdump.o
	$(CC) $(CFLAGS) -o $@ -Wl,-Map,main.map,--gc-sections $^

pcsim: CC=gcc -std=gnu99
pcsim: CFLAGS=-Wall -Wextra -DON_PC -D_GNU_SOURCE -O2 -g
pcsim: main.o seq.o pwm.o enc28j60_pc.o arp.o autoip.o timer_pc.o event_pc.o ip.o icmp.o udp.o dhcp.o eth.o netbios.o light.o tcp.o eprom.o irq.o
	$(CC) $(CFLAGS) -o $@ -Wl,-Map,main.map,--gc-sections $^ -lpcap -lpthread

depend:
	makedepend -f Makefile.make -Y . -- $(CFLAGS) -- *.c

size:
	tail -n 1 size.log

load: main.hex
	avrdude -p m168 -P /dev/ttyUSB0 -c stk500v2 -e -U flash:w:$^

loadeeprom: eeprom.hex main.hex
	avrdude -p m168 -P com5 -c stk500v2    -U hfuse:w:0xdf:m
	avrdude -p m168 -P com5 -c stk500v2 -e -U eeprom:w:eeprom.hex -U flash:w:main.hex
	avrdude -p m168 -P com5 -c stk500v2    -U hfuse:w:0xd7:m

%.s: %.c
	$(CC) $(CFLAGS) -s -o $@ $^

rdfuses:
	avrdude -p m168 -P com5 -c stk500v2 -e -U efuse:r:-:h -U hfuse:r:-:h -U lfuse:r:-:h

clean:
	rm -f *.o *.map *.elf

distclean: clean
	rm -f *.map *.hex

.PHONY: load loadeeprom size depend rdfuses clean distclean

# DO NOT DELETE

arp.o: main.h enc28j60.h eth.h arp.h ip.h
autoip.o: enc28j60.h main.h eth.h autoip.h ip.h event.h timer.h arp.h
dhcp.o: enc28j60.h main.h eth.h autoip.h ip.h event.h timer.h dhcp.h udp.h
dhcp.o: arp.h
enc28j60.o: enc28j60.h main.h eth.h enc28j60_reg.h event.h
enc28j60_pc.o: enc28j60.h main.h eth.h event.h
enc28j60_stub.o: enc28j60.h main.h eth.h
eprom.o: enc28j60.h main.h eth.h timer.h eprom.h seq.h ip.h pwm.h udp.h
eth.o: enc28j60.h main.h eth.h event.h timer.h arp.h ip.h
event_pc.o: main.h event.h
icmp.o: enc28j60.h main.h eth.h icmp.h arp.h ip.h
ip.o: enc28j60.h main.h eth.h icmp.h arp.h ip.h tcp.h udp.h
irq.o: irq.h
light.o: enc28j60.h main.h eth.h eprom.h seq.h ip.h light.h udp.h
main.o: stackmon.h enc28j60.h main.h eth.h autoip.h ip.h timer.h event.h
main.o: light.h eprom.h seq.h udp.h dhcp.h arp.h tcp.h
memdump.o: enc28j60.h main.h eth.h memdump.h ip.h udp.h
netbios.o: enc28j60.h main.h eth.h netbios.h udp.h ip.h
pwm.o: main.h pwm.h
seq.o: enc28j60.h main.h eth.h light.h eprom.h seq.h ip.h udp.h timer.h
seq.o: event.h tcp.h pwm.h
stackmon.o: stackmon.h
tcp.o: main.h ip.h eth.h tcp.h event.h timer.h enc28j60.h udp.h
timer.o: eprom.h seq.h main.h ip.h eth.h timer.h event.h irq.h
timer_pc.o: timer.h event.h eprom.h seq.h main.h ip.h eth.h
udp.o: enc28j60.h main.h eth.h netbios.h udp.h ip.h memdump.h eprom.h seq.h
udp.o: light.h dhcp.h
