/***************************************************************************
 * eprom.h: Eprom image types for pre-defined light sequences.
 * Copyright (C) 2007 Michael C McTernan,
 *    Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef EPROM_H
#define EPROM_H

/**************************************************************************
 * Nested Include Files
 **************************************************************************/

#if !defined(ON_PC)
#include <avr/eeprom.h>
#else
#define EEMEM
#endif
#include <stdint.h>
#include "seq.h"
#include "ip.h"

/**************************************************************************
 * Manifest Constants
 **************************************************************************/

/**************************************************************************
 * Type Definitions
 **************************************************************************/


typedef enum EpromLightSequenceTag
{
    EPROM_LIGHT_POST,           /**< Power on self test. */
    EPROM_LIGHT_ETH_UP,
    EPROM_LIGHT_ETH_DOWN,
    EPROM_LIGHT_DHCP_START,
    EPROM_LIGHT_DHCP_CONFIGURED,
    EPROM_LIGHT_AUTOIP_START,
    EPROM_LIGHT_AUTOIP_CONFIGURED,
    EPROM_LIGHT_START,
    NUM_EPROM_LIGHT
}
EpromLightSequence;


typedef struct EpromImage
{
    /** The number of predefined sequences. */
    uint8_t    predefNum;

    /** Offsets to predefined sequences.
     * Note that an extra element is defined to store the length of the data
     * array, \a predef[].
     */
    uint8_t    predefOffset[NUM_EPROM_LIGHT + 1];

    /** Array of pre-defined sequences. */
    SeqCommand predef[];
}
EpromImage;


/**************************************************************************
 * Global Data
 **************************************************************************/

/** The EPROM image.
 */
extern EpromImage EEMEM epromImage;

/**************************************************************************
 * Global Functions
 **************************************************************************/

void EpromProcessPacket     (const ipparam_t *ipp);
void EpromAllowReprogramming(boolean_t allow);

#endif /* EPROM_H */


/* END OF FILE */
