/***************************************************************************
 * ip.h: IP layer for ATmega88.
 * Copyright (C) 2007 Michael C McTernan,
 *    Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef IP_H
#define IP_H

/**************************************************************************
 * Include Files
 **************************************************************************/

#include <stdint.h>
#include "eth.h"

/**************************************************************************
 * Manifest Constants
 **************************************************************************/

#define IP_PROT_ICMP  0x01
#define IP_PROT_TCP   0x06
#define IP_PROT_UDP   0x11

/**************************************************************************
 * Types
 **************************************************************************/

/** Type for an IP address.
 */
typedef union
{
    uint8_t  b[4];
    uint32_t l;
}
ipaddr_t;

/** Type giving useful IP header parameters.
 */
typedef struct
{
    uint8_t  hLen;          /**< Header length in bytes. */
    uint16_t payloadLen;    /**< Payload length in bytes. */
    ipaddr_t sAddr;         /**< Packet source IP address. */
}
ipparam_t;

/**************************************************************************
 * Global Variables
 **************************************************************************/

extern const ipaddr_t ipZero;
extern const ipaddr_t ipBCast;
extern       ipaddr_t ipHost;
extern       ipaddr_t ipSCast;

/**************************************************************************
 * Prototypes
 **************************************************************************/

void      IpProcessPacket(void);

boolean_t IpWriteHeader(const ipaddr_t *sAddr,
                        const ipaddr_t *dAddr,
                        const int16_t   tLength,
                        uint8_t         prot);

uint16_t  IpChecksum   (uint32_t *const state, uint16_t n, const void *data);

void      IpSetHostAddr(const ipaddr_t *ipaddr, uint8_t maskLen);

void      IpSetRoute   (const ipaddr_t *gateway,
                        const ipaddr_t *subnet,
                        const uint8_t   maskbits);

void      IpReset(void);

#endif

/* END OF FILE */
