/***************************************************************************
 * irq.c: Interrupt enabling/disabling.
 * Copyright (C) 2007 Michael C McTernan,
 *    Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ***************************************************************************/

/**************************************************************************
 * Includes
 **************************************************************************/

#if !defined(ON_PC)
#include <avr/interrupt.h>
#endif
#include <stdint.h>
#include "irq.h"

/**************************************************************************
 * Manfest Constants
 **************************************************************************/

/**************************************************************************
 * Macros
 **************************************************************************/

/**************************************************************************
 * Types
 **************************************************************************/

/**************************************************************************
 * Local Variables
 **************************************************************************/

/** Nested disable count for interrupts.
 */
static uint8_t irqDisableCount = 0;

/**************************************************************************
 * Local Functions
 **************************************************************************/

/**************************************************************************
 * Global Functions
 **************************************************************************/

/** Disable interrupts.
 */
void IrqDisable(void)
{
#ifndef ON_PC
    cli();
#endif
    irqDisableCount++;
}

/** Conditionally enable interrupts.
 */
void IrqEnable(void)
{
    if(--irqDisableCount == 0)
    {
#ifndef ON_PC
        sei();
#endif
    }
}

/* END OF FILE */
