/***************************************************************************
 * udp.h: UPD layer for ATmega88.
 * Copyright (C) 2007 Michael C McTernan,
 *    Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef UDP_H
#define UDP_H

/**************************************************************************
 * Include Files
 **************************************************************************/

#include <stdint.h>
#include "udp.h"
#include "ip.h"

/**************************************************************************
 * Manifest Constants
 **************************************************************************/

/** Length of a UDP header in bytes.
 */
#define UDP_HDR_LEN_BYTES        8

#define UDP_PORT_BOOTPS         67  /* Server */
#define UDP_PORT_BOOTPC         68  /* Client */
#define UDP_PORT_NETBIOS_NS    137
#define UDP_PORT_LIGHTCONTROL 8888
#define UDP_PORT_LIGHTUPDATE  8889
#define UDP_PORT_MEMDUMP      8890

/**************************************************************************
 * Types
 **************************************************************************/

typedef struct
{
    uint16_t sport;      /**< Source port. */
    uint16_t payloadLen; /**< Length of datagram payload. */
}
udpparam_t;

/**************************************************************************
 * Prototypes
 **************************************************************************/

void      UdpProcessPacket(const ipparam_t *ipp);

boolean_t UdpWriteHeader  (const ipaddr_t *sAddr,
                           const ipaddr_t *dAddr,
                           uint16_t        sport,
                           uint16_t        dport,
                           uint16_t        length);

void      UdpDebug        (uint8_t v1, uint8_t v2, uint8_t v3,
                           uint8_t v4, uint8_t v5, uint8_t v6);


#endif

/* END OF FILE */
